#include <stdarg.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdio.h>

#include "runtime.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>

#define DbgBreak DebugBreak()
#elif defined(__GNUC__)
#define DbgBreak __builtin_trap()
#else
/* Not available */
#define DbgBreak 
#endif

/* TODO(Kevin): Log to file, show error message box, ... */

static bool DisplayErrorBox(const char *text) {
#ifdef _WIN32
    WCHAR msg[256];
    vyUTF8ToWStr(text, msg, VY_ARRAY_COUNT(msg));

#ifdef VY_ERROR_REPORT_DEBUGBREAK
    return MessageBoxW(NULL, msg, L"Error", MB_OKCANCEL | MB_ICONERROR) == IDCANCEL;
#else
    MessageBoxW(NULL, msg, L"Error", MB_ICONERROR);
    return false;
#endif /* VY_ERROR_REPORT_DEBUGBREAK */
#else
    return false;
#endif
}

static void LogOut(const char *text) {
#ifdef _WIN32
    WCHAR msg[256];
    vyUTF8ToWStr(text, msg, VY_ARRAY_COUNT(msg));
    OutputDebugStringW(msg);
#endif
    fprintf(stderr, "%s", text);
}

VY_DLLEXPORT void vyReportError(const char *subsystem, const char *fmt, ...) {
    char buf[256];

    int at = snprintf(buf, VY_ARRAY_COUNT(buf) - 1, "[%s] ", subsystem);
    va_list ap;
    va_start(ap, fmt);
    at += vsnprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, fmt, ap);
    va_end(ap);
    at += snprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, "\n");

    LogOut(buf);
    if (DisplayErrorBox(buf)) {
        DbgBreak;
    }
}

VY_DLLEXPORT void vyLog(const char *subsystem, const char *fmt, ...) {
    char buf[256];

    int at = snprintf(buf, VY_ARRAY_COUNT(buf) - 1, "[%s] ", subsystem);
    va_list ap;
    va_start(ap, fmt);
    at += vsnprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, fmt, ap);
    va_end(ap);
    at += snprintf(&buf[at], VY_ARRAY_COUNT(buf) - at - 1, "\n");

    LogOut(buf);
}