#include "runtime.h"
#include "config.h"
#include "aio.h"
#include "file_tab.h"
#include "buffer_manager.h"
#include "uidtab.h"

extern vy_cvar rt_Renderer;
extern vy_cvar rt_Fullscreen;
extern vy_cvar rt_WindowWidth;
extern vy_cvar rt_WindowHeight;
extern vy_cvar rt_BufferManagerMemory;
extern vy_cvar rt_FileTabCapacity;
extern vy_cvar rt_MaxConcurrentAsyncIO;
extern vy_cvar rt_AssetCacheSize;

void RegisterRuntimeCVars(void) {
    vyRegisterCVAR(&rt_Renderer);
    vyRegisterCVAR(&rt_Fullscreen);
    vyRegisterCVAR(&rt_WindowWidth);
    vyRegisterCVAR(&rt_WindowHeight);
    vyRegisterCVAR(&rt_BufferManagerMemory);
    vyRegisterCVAR(&rt_FileTabCapacity);
    vyRegisterCVAR(&rt_MaxConcurrentAsyncIO);
    vyRegisterCVAR(&rt_AssetCacheSize);
}

extern void SetMainThreadId(void);

extern vy_result InitBufferManager(void);
extern void ShutdownBufferManager(void);
extern vy_result InitFileTab(void);
extern void ShutdownFileTab(void);
extern vy_result InitAIO(void);
extern void ShutdownAIO(void);
extern vy_result InitAssetCache(void);
extern void ShutdownAssetCache(void);

extern vy_result LoadUIDTable(void);
extern void ReleaseUIDTable(void);
extern vy_result LoadAssetDependencies(void); 
extern void ReleaseAssetDependencies(void);
extern vy_result LoadPackageNames(void);

VY_DLLEXPORT vy_result vyInitRuntime(void) {
    SetMainThreadId();
    RegisterRuntimeCVars();

    vy_result res;
    if ((res = InitBufferManager()) != VY_SUCCESS) {
        vyReportError("BUFFERMGR", "Init failed.");
        return res;
    }

    if ((res = InitFileTab()) != VY_SUCCESS) {
        vyReportError("FTAB", "Init failed.");
        return res;
    }

    if ((res = InitAIO()) != VY_SUCCESS) {
        vyReportError("AIO", "Init failed.");
        return res;
    }

    if ((res = InitAssetCache()) != VY_SUCCESS) {
        vyReportError("ASSETCACHE", "Init failed.");
        return res;
    }

    if ((res = LoadUIDTable()) != VY_SUCCESS) {
        vyLog("CORE", "LoadUIDTable returned result: %u", res);
    }

    if ((res = LoadAssetDependencies()) != VY_SUCCESS) {
        vyReportError("ASSETDEP", "Init failed.");
        return res;
    }

    if ((res = LoadPackageNames()) != VY_SUCCESS) {
        vyLog("CORE", "LoadPackageNames returned result: %u", res);
    }

    return VY_SUCCESS;
}

VY_DLLEXPORT void vyShutdownRuntime(void) {
    ReleaseAssetDependencies();
    ReleaseUIDTable();
    ShutdownAssetCache();
    ShutdownAIO();
    ShutdownFileTab();
    ShutdownBufferManager();
}