#ifndef VY_THREADING_H
#define VY_THREADING_H

/* platform independent threading */

#include <stdbool.h>

#include "runtime.h"

/* Mutexes */

typedef struct vy_mutex_s vy_mutex;

VY_DLLEXPORT vy_mutex *vyCreateMutex(void);

VY_DLLEXPORT void vyDestroyMutex(vy_mutex *mutex);

VY_DLLEXPORT bool vyLockMutex(vy_mutex *mutex);

VY_DLLEXPORT bool vyUnlockMutex(vy_mutex *mutex);

/* Condition variables */

typedef struct vy_condition_var_s vy_condition_var;

VY_DLLEXPORT vy_condition_var *vyCreateConditionVar(void);

VY_DLLEXPORT void vyDestroyConditionVar(vy_condition_var *var);

VY_DLLEXPORT void vyLockConditionVar(vy_condition_var *var);

VY_DLLEXPORT void vyUnlockConditionVar(vy_condition_var *var, bool signal);

/* The condition variable must be locked by the thread! */
VY_DLLEXPORT void vyWaitOnConditionVar(vy_condition_var *var);

/* Read-Write Lock */
typedef struct {
    volatile int reader_count;
    vy_condition_var *cond;
} vy_rwlock;

typedef struct {
    bool ok;
    vy_rwlock lock;
} vy_create_rwlock_result;

VY_DLLEXPORT vy_create_rwlock_result vyCreateRWLock(void);

VY_DLLEXPORT void vyDestroyRWLock(vy_rwlock *lock);

VY_DLLEXPORT void vyLockRead(vy_rwlock *lock);

VY_DLLEXPORT void vyUnlockRead(vy_rwlock *lock);

VY_DLLEXPORT void vyLockWrite(vy_rwlock *lock);

VY_DLLEXPORT void vyUnlockWrite(vy_rwlock *lock);

/* Threads */

typedef struct vy_thread_s vy_thread;

typedef void vy_thread_entry_fn(void *param);

VY_DLLEXPORT vy_thread *vySpawnThread(vy_thread_entry_fn *entry, void *param, const char *name);

VY_DLLEXPORT void vyJoinThread(vy_thread *thread);

VY_DLLEXPORT unsigned int vyGetCPUCoreCount(void);

typedef uint32_t vy_thread_id;

VY_DLLEXPORT vy_thread_id vyGetCurrentThreadId(void);

VY_DLLEXPORT bool vyIsMainThread(void);

#endif
