#include "threading.h"

#include <assert.h>

/* Based on: https://eli.thegreenplace.net/2019/implementing-reader-writer-locks/ */

VY_DLLEXPORT vy_create_rwlock_result vyCreateRWLock(void) {
    vy_create_rwlock_result res;
    res.lock.reader_count = 0;
    res.lock.cond         = vyCreateConditionVar();
    res.ok                = res.lock.cond != NULL;
    return res;
}

VY_DLLEXPORT void vyDestroyRWLock(vy_rwlock *lock) {
    vyDestroyConditionVar(lock->cond);
    lock->cond         = NULL;
    lock->reader_count = 0;
}

VY_DLLEXPORT void vyLockRead(vy_rwlock *lock) {
    vyLockConditionVar(lock->cond);
    ++lock->reader_count;
    vyUnlockConditionVar(lock->cond, false);
}

VY_DLLEXPORT void vyUnlockRead(vy_rwlock *lock) {
    vyLockConditionVar(lock->cond);
    assert(lock->reader_count > 0);
    --lock->reader_count;
    bool signal = lock->reader_count == 0;
    vyUnlockConditionVar(lock->cond, signal);
}

VY_DLLEXPORT void vyLockWrite(vy_rwlock *lock) {
    vyLockConditionVar(lock->cond);
    while (lock->reader_count > 0)
        vyWaitOnConditionVar(lock->cond);
}

VY_DLLEXPORT void vyUnlockWrite(vy_rwlock *lock) {
    vyUnlockConditionVar(lock->cond, true);
}
